-- frontier_silo.lua
-- Jan 2018
-- My take on frontier silos for my Oarc scenario

require("config")
require("DE333/DE333_config")
require("lib/oarc_utils")

local math_random = math.random
local black_market3_items = require "DE333.market.black_market3_items"

--------------------------------------------------------------------------------
-- Frontier style rocket silo stuff
--------------------------------------------------------------------------------

-- This creates a random silo position, stored to global.BlackMarket3Position
-- It uses the config setting global.ocfg.black_market1_chunk_distance and spawns the
-- silo somewhere on a circle edge with radius using that distance.

function SetRandomBlackMarket3Position(num_blackmarket3)
    if (global.BlackMarket3Position == nil) then

        global.BlackMarket3Position = {}

        random_angle_offset = (math.random (16, 32)) + (math.random (32, 64))

        for i=1,num_blackmarket3 do
            theta = (math.random (8, 20) / num_blackmarket3);
            angle = (theta * i) + random_angle_offset * math.random (15, 30);

            tx = (global.ocfg.black_market3_chunk_distance * CHUNK_SIZE * (math.random (8, 20)) / (math.random (8, 20)) * math.cos(angle))
            ty = (global.ocfg.black_market3_chunk_distance * CHUNK_SIZE * (math.random (12, 24)) / (math.random (12, 24)) * math.sin(angle))

            table.insert(global.BlackMarket3Position, {x=math.floor(tx), y=math.floor(ty)})

            log("Black Market 3 position: " .. tx .. ", " .. ty .. ", " .. angle)
        end
    end
end

-- Create a rocket silo at the specified positionmmmm
-- Also makes sure tiles and entities are cleared if required.
local function CreateBlackMarket3(surface, BlackMarket3Position, force)

    -- Delete any entities beneath the silo?
    for _, entity in pairs(surface.find_entities_filtered{area = {{BlackMarket3Position.x-5,
                                                                    BlackMarket3Position.y-6},
                                                                    {BlackMarket3Position.x+6,
                                                                    BlackMarket3Position.y+6}}}) do
        entity.destroy()
    end

    -- Remove nearby enemies again
    for _, entity in pairs(surface.find_entities_filtered{area = {{BlackMarket3Position.x-(CHUNK_SIZE*4),
                                                                    BlackMarket3Position.y-(CHUNK_SIZE*4)},
                                                                    {BlackMarket3Position.x+(CHUNK_SIZE*4),
                                                                    BlackMarket3Position.y+(CHUNK_SIZE*4)}}, force = "enemy"}) do
        entity.destroy()
    end

    -- Set tiles below the silo
    tiles = {}
    i = 1
    for dx = -5,5 do
        for dy = -5,5 do
            tiles[i] = {name = "concrete", position = {BlackMarket3Position.x+dx, BlackMarket3Position.y+dy}}
            i=i+1
        end
    end
    surface.set_tiles(tiles, true)

    -- Create indestructible silo and assign to a force
    local BlackMarket3 = surface.create_entity{name = "market", position = {BlackMarket3Position.x+0.5, BlackMarket3Position.y}, force = force}
    BlackMarket3.destructible = false
    BlackMarket3.minable = false

	for _, item in pairs(black_market3_items.spawn) do
			BlackMarket3.add_market_item(item)
	end

    -- Make silo safe from being removed by regrowth
    -- if global.ocfg.enable_regrowth then
        OarcRegrowthOffLimits(BlackMarket3Position, 5)
    -- end


	if (global.ocfg.black_market3_wall) then
        market3wall(surface, BlackMarket3Position, game.forces[global.ocfg.main_force])
    end
	
	if (global.ocfg.black_market3_light) then
        market3light(surface, BlackMarket3Position, game.forces[global.ocfg.main_force])
    end
	
	if (global.ocfg.black_market3_biter) then
        market3biter(surface, BlackMarket3Position, game.forces[global.ocfg.main_force])
    end
	
	if (global.ocfg.black_market3_wreckage) then
        market3wreckage(surface, BlackMarket3Position, game.forces[global.ocfg.main_force])
    end
	
	if (global.ocfg.black_market3_ship) then
            GenerateLootShipBM3(surface, BlackMarket3Position, game.forces[global.ocfg.main_force])
        end
        
end

-- Generates all rocket silos, should be called after the areas are generated
-- Includes a crop circle
function GenerateAllBlackMarket3(surface)                       
    
    -- Create each silo in the list
    for idx,BlackMarket3Pos in pairs(global.BlackMarket3Position) do
        CreateBlackMarket3(surface, BlackMarket3Pos, global.ocfg.main_force)
    end
end

-- Generate clean land and trees around silo area on chunk generate event
function GenerateBlackMarket3Chunk(event)

    -- Silo generation can take awhile depending on the number of silos.
    if (game.tick < global.ocfg.black_market3_count*10*TICKS_PER_SECOND) then
        local surface = event.surface
        local chunkArea = event.area

        local chunkAreaCenter = {x=chunkArea.left_top.x+(CHUNK_SIZE/2),
                                 y=chunkArea.left_top.y+(CHUNK_SIZE/2)}
        
        for idx,BlackMarket3Pos in pairs(global.BlackMarket3Position) do
            local safeArea = {left_top=
                                {x=BlackMarket3Pos.x-(CHUNK_SIZE*4),
                                 y=BlackMarket3Pos.y-(CHUNK_SIZE*4)},
                              right_bottom=
                                {x=BlackMarket3Pos.x+(CHUNK_SIZE*4),
                                 y=BlackMarket3Pos.y+(CHUNK_SIZE*4)}}
                                     

            -- Clear enemies directly next to the rocket
            if CheckIfInArea(chunkAreaCenter,safeArea) then

                -- Remove trees/resources inside the spawn area
                RemoveInCircle(surface, chunkArea, "tree", BlackMarket3Pos, CHUNK_SIZE*2)
                RemoveInCircle(surface, chunkArea, "resource", BlackMarket3Pos, CHUNK_SIZE*2)
                RemoveInCircle(surface, chunkArea, "cliff", BlackMarket3Pos, CHUNK_SIZE*2)
                RemoveDecorationsArea(surface, chunkArea)

                -- Create rocket silo
                CreateCropOctagon4(surface, BlackMarket3Pos, chunkArea, CHUNK_SIZE)
            end
        end
    end
end

-- Generate chunks where we plan to place the rocket silos.
function GenerateBlackMarket3Areas(surface)
    for idx,BlackMarket3Pos in pairs(global.BlackMarket3Position) do
        surface.request_to_generate_chunks({BlackMarket3Pos.x, BlackMarket3Pos.y}, 3)
    end
	if (global.ocfg.black_market3_vision) then
        ChartBlackMarket3Areas(surface, game.forces[global.ocfg.main_force])
    end
end

-- Chart chunks where we plan to place the rocket silos.
function ChartBlackMarket3Areas(surface, force)
    for idx,BlackMarket3Pos in pairs(global.BlackMarket3Position) do
        force.chart(surface, {{BlackMarket3Pos.x-(CHUNK_SIZE*2),
                                BlackMarket3Pos.y-(CHUNK_SIZE*2)},
                                {BlackMarket3Pos.x+(CHUNK_SIZE*2),
                                BlackMarket3Pos.y+(CHUNK_SIZE*2)}})
    end
end

global.black_market3_generated = false
function DelayedBlackMarket3CreationOnTick(surface)

    -- Delay the creation of the silos so we place them on already generated lands.
    if (not global.black_market3_generated and (game.tick >= global.ocfg.frontier_silo_count*16*TICKS_PER_SECOND)) then
        log("Black Market 3 generated!")
        global.black_market3_generated = true
        GenerateAllBlackMarket3(surface)
    end

end 

function GenerateLootShipBM3(surface, BlackMarket3Pos, force)
    
local chest_raffle_BLACKMARKET3 = {}
	local chest_loot_BLACKMARKET3 = {			
		--{{name = "submachine-gun", count = math_random(1,3)}, weight = 3, evolution_min = 0.0, evolution_max = 0.1},		
		--{{name = "slowdown-capsule", count = math_random(16,32)}, weight = 1, evolution_min = 0.3, evolution_max = 0.7},
		--{{name = "poison-capsule", count = math_random(16,32)}, weight = 3, evolution_min = 0.3, evolution_max = 1},		
		--{{name = "uranium-cannon-shell", count = math_random(16,32)}, weight = 5, evolution_min = 0.6, evolution_max = 1},
		--{{name = "cannon-shell", count = math_random(16,32)}, weight = 5, evolution_min = 0.4, evolution_max = 0.7},
		--{{name = "explosive-uranium-cannon-shell", count = math_random(16,32)}, weight = 5, evolution_min = 0.6, evolution_max = 1},
		--{{name = "explosive-cannon-shell", count = math_random(16,32)}, weight = 5, evolution_min = 0.4, evolution_max = 0.8},
		--{{name = "shotgun", count = 1}, weight = 2, evolution_min = 0.0, evolution_max = 0.2},
		--{{name = "shotgun-shell", count = math_random(16,32)}, weight = 5, evolution_min = 0.0, evolution_max = 0.2},
		--{{name = "combat-shotgun", count = 1}, weight = 3, evolution_min = 0.3, evolution_max = 0.8},
		--{{name = "piercing-shotgun-shell", count = math_random(16,32)}, weight = 10, evolution_min = 0.2, evolution_max = 1},
		--{{name = "flamethrower", count = 1}, weight = 3, evolution_min = 0.3, evolution_max = 0.6},
		--{{name = "flamethrower-ammo", count = math_random(16,32)}, weight = 5, evolution_min = 0.3, evolution_max = 1},
		--{{name = "rocket-launcher", count = 1}, weight = 3, evolution_min = 0.2, evolution_max = 0.6},
		--{{name = "rocket", count = math_random(16,32)}, weight = 5, evolution_min = 0.2, evolution_max = 0.7},		
		--{{name = "explosive-rocket", count = math_random(16,32)}, weight = 5, evolution_min = 0.3, evolution_max = 1},
		--{{name = "land-mine", count = math_random(16,32)}, weight = 5, evolution_min = 0.2, evolution_max = 0.7},
		--{{name = "grenade", count = math_random(16,32)}, weight = 5, evolution_min = 0.0, evolution_max = 0.5},
		--{{name = "cluster-grenade", count = math_random(16,32)}, weight = 5, evolution_min = 0.4, evolution_max = 1},
		--{{name = "firearm-magazine", count = math_random(32,128)}, weight = 5, evolution_min = 0, evolution_max = 0.3},
		--{{name = "piercing-rounds-magazine", count = math_random(32,128)}, weight = 5, evolution_min = 0.1, evolution_max = 0.8},
		{{name = "uranium-rounds-magazine", count = math_random(32,128)}, weight = 5, evolution_min = 0.5, evolution_max = 1},
		--{{name = "railgun", count = 1}, weight = 1, evolution_min = 0.2, evolution_max = 1},
		--{{name = "railgun-dart", count = math_random(16,32)}, weight = 3, evolution_min = 0.2, evolution_max = 0.7},
		--{{name = "defender-capsule", count = math_random(8,16)}, weight = 2, evolution_min = 0.0, evolution_max = 0.7},
		--{{name = "distractor-capsule", count = math_random(8,16)}, weight = 2, evolution_min = 0.2, evolution_max = 1},
		--{{name = "destroyer-capsule", count = math_random(8,16)}, weight = 2, evolution_min = 0.3, evolution_max = 1},
		--{{name = "atomic-bomb", count = math_random(8,16)}, weight = 1, evolution_min = 0.3, evolution_max = 1},		
		--{{name = "light-armor", count = 1}, weight = 3, evolution_min = 0, evolution_max = 0.1},		
		--{{name = "heavy-armor", count = 1}, weight = 3, evolution_min = 0.1, evolution_max = 0.3},
		--{{name = "modular-armor", count = 1}, weight = 2, evolution_min = 0.2, evolution_max = 0.6},
		{{name = "power-armor", count = 1}, weight = 2, evolution_min = 0.4, evolution_max = 1},
		--{{name = "power-armor-mk2", count = 1}, weight = 1, evolution_min = 0.9, evolution_max = 1},
		{{name = "battery-equipment", count = 1}, weight = 2, evolution_min = 0.3, evolution_max = 0.7},
		--{{name = "battery-mk2-equipment", count = 1}, weight = 2, evolution_min = 0.6, evolution_max = 1},
		{{name = "belt-immunity-equipment", count = 1}, weight = 1, evolution_min = 0.3, evolution_max = 1},
		{{name = "solar-panel-equipment", count = math_random(1,4)}, weight = 5, evolution_min = 0.3, evolution_max = 0.8},
		{{name = "discharge-defense-equipment", count = 1}, weight = 1, evolution_min = 0.5, evolution_max = 0.8},
		{{name = "energy-shield-equipment", count = math_random(1,2)}, weight = 2, evolution_min = 0.3, evolution_max = 0.8},
		{{name = "energy-shield-mk2-equipment", count = 1}, weight = 2, evolution_min = 0.7, evolution_max = 1},
		{{name = "exoskeleton-equipment", count = 1}, weight = 1, evolution_min = 0.3, evolution_max = 1},
		{{name = "fusion-reactor-equipment", count = 1}, weight = 1, evolution_min = 0.5, evolution_max = 1},
		{{name = "night-vision-equipment", count = 1}, weight = 1, evolution_min = 0.3, evolution_max = 0.8},
		{{name = "personal-laser-defense-equipment", count = 1}, weight = 2, evolution_min = 0.5, evolution_max = 1},
		{{name = "exoskeleton-equipment", count = 1}, weight = 1, evolution_min = 0.3, evolution_max = 1},
						
		
		--{{name = "iron-gear-wheel", count = math_random(80,100)}, weight = 3, evolution_min = 0.0, evolution_max = 0.3},
		--{{name = "copper-cable", count = math_random(100,200)}, weight = 3, evolution_min = 0.0, evolution_max = 0.3},
		--{{name = "engine-unit", count = math_random(16,32)}, weight = 2, evolution_min = 0.1, evolution_max = 0.5},
		--{{name = "electric-engine-unit", count = math_random(16,32)}, weight = 2, evolution_min = 0.4, evolution_max = 0.8},
		--{{name = "battery", count = math_random(50,150)}, weight = 2, evolution_min = 0.3, evolution_max = 0.8},
		--{{name = "advanced-circuit", count = math_random(50,150)}, weight = 3, evolution_min = 0.4, evolution_max = 1},
		--{{name = "electronic-circuit", count = math_random(50,150)}, weight = 3, evolution_min = 0.0, evolution_max = 0.4},
		--{{name = "processing-unit", count = math_random(50,150)}, weight = 3, evolution_min = 0.7, evolution_max = 1},
		--{{name = "explosives", count = math_random(40,50)}, weight = 10, evolution_min = 0.0, evolution_max = 1},
		--{{name = "lubricant-barrel", count = math_random(4,10)}, weight = 1, evolution_min = 0.3, evolution_max = 0.5},
		--{{name = "rocket-fuel", count = math_random(4,10)}, weight = 2, evolution_min = 0.3, evolution_max = 0.7},
		--{{name = "computer", count = 1}, weight = 2, evolution_min = 0, evolution_max = 1},
		--{{name = "steel-plate", count = math_random(25,75)}, weight = 2, evolution_min = 0.1, evolution_max = 0.3},
		{{name = "nuclear-fuel", count = 1}, weight = 2, evolution_min = 0.7, evolution_max = 1},
				
		--{{name = "burner-inserter", count = math_random(8,16)}, weight = 3, evolution_min = 0.0, evolution_max = 0.1},
		--{{name = "inserter", count = math_random(8,16)}, weight = 3, evolution_min = 0.0, evolution_max = 0.4},
		--{{name = "long-handed-inserter", count = math_random(8,16)}, weight = 3, evolution_min = 0.0, evolution_max = 0.4},		
		--{{name = "fast-inserter", count = math_random(8,16)}, weight = 3, evolution_min = 0.1, evolution_max = 1},
		--{{name = "filter-inserter", count = math_random(8,16)}, weight = 1, evolution_min = 0.2, evolution_max = 1},		
		--{{name = "stack-filter-inserter", count = math_random(4,8)}, weight = 1, evolution_min = 0.4, evolution_max = 1},
		--{{name = "stack-inserter", count = math_random(4,8)}, weight = 3, evolution_min = 0.3, evolution_max = 1},				
		--{{name = "small-electric-pole", count = math_random(16,24)}, weight = 3, evolution_min = 0.0, evolution_max = 0.3},
		--{{name = "medium-electric-pole", count = math_random(8,16)}, weight = 3, evolution_min = 0.2, evolution_max = 1},
		--{{name = "big-electric-pole", count = math_random(4,8)}, weight = 3, evolution_min = 0.3, evolution_max = 1},
		{{name = "substation", count = math_random(2,4)}, weight = 3, evolution_min = 0.5, evolution_max = 1},
		--{{name = "wooden-chest", count = math_random(16,24)}, weight = 3, evolution_min = 0.0, evolution_max = 0.2},
		--{{name = "iron-chest", count = math_random(4,8)}, weight = 3, evolution_min = 0.1, evolution_max = 0.4},
		--{{name = "steel-chest", count = math_random(4,8)}, weight = 3, evolution_min = 0.3, evolution_max = 1},		
		--{{name = "small-lamp", count = math_random(16,32)}, weight = 3, evolution_min = 0.1, evolution_max = 0.3},
		--{{name = "rail", count = math_random(25,75)}, weight = 3, evolution_min = 0.1, evolution_max = 0.6},
		--{{name = "assembling-machine-1", count = math_random(2,4)}, weight = 3, evolution_min = 0.0, evolution_max = 0.3},
		--{{name = "assembling-machine-2", count = math_random(2,4)}, weight = 3, evolution_min = 0.2, evolution_max = 0.8},
		{{name = "assembling-machine-3", count = math_random(1,2)}, weight = 3, evolution_min = 0.5, evolution_max = 1},
		{{name = "accumulator", count = math_random(4,8)}, weight = 3, evolution_min = 0.4, evolution_max = 1},
		--{{name = "offshore-pump", count = math_random(1,3)}, weight = 2, evolution_min = 0.0, evolution_max = 0.1},
		{{name = "beacon", count = math_random(1,2)}, weight = 3, evolution_min = 0.7, evolution_max = 1},
		--{{name = "boiler", count = math_random(4,8)}, weight = 3, evolution_min = 0.0, evolution_max = 0.3},
		--{{name = "steam-engine", count = math_random(2,4)}, weight = 3, evolution_min = 0.0, evolution_max = 0.5},
		--{{name = "steam-turbine", count = math_random(1,2)}, weight = 2, evolution_min = 0.6, evolution_max = 1},
		--{{name = "nuclear-reactor", count = 1}, weight = 1, evolution_min = 0.6, evolution_max = 1},
		{{name = "centrifuge", count = math_random(1,2)}, weight = 1, evolution_min = 0.6, evolution_max = 1},
		--{{name = "heat-pipe", count = math_random(4,8)}, weight = 2, evolution_min = 0.5, evolution_max = 1},
		--{{name = "heat-exchanger", count = math_random(2,4)}, weight = 2, evolution_min = 0.5, evolution_max = 1},
		--{{name = "arithmetic-combinator", count = math_random(8,16)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		--{{name = "constant-combinator", count = math_random(8,16)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		--{{name = "decider-combinator", count = math_random(8,16)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		--{{name = "power-switch", count = math_random(1,2)}, weight = 1, evolution_min = 0.1, evolution_max = 1},		
		--{{name = "programmable-speaker", count = math_random(4,8)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		--{{name = "green-wire", count = math_random(50,99)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		--{{name = "red-wire", count = math_random(50,99)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		--{{name = "chemical-plant", count = math_random(1,3)}, weight = 3, evolution_min = 0.3, evolution_max = 1},
		--{{name = "burner-mining-drill", count = math_random(2,4)}, weight = 3, evolution_min = 0.0, evolution_max = 0.2},
		--{{name = "electric-mining-drill", count = math_random(2,4)}, weight = 3, evolution_min = 0.2, evolution_max = 0.6},		
		{{name = "express-transport-belt", count = math_random(25,75)}, weight = 3, evolution_min = 0.5, evolution_max = 1},
		{{name = "express-underground-belt", count = math_random(4,8)}, weight = 3, evolution_min = 0.5, evolution_max = 1},		
		{{name = "express-splitter", count = math_random(2,4)}, weight = 3, evolution_min = 0.5, evolution_max = 1},
		--{{name = "fast-transport-belt", count = math_random(25,75)}, weight = 3, evolution_min = 0.2, evolution_max = 0.7},
		--{{name = "fast-underground-belt", count = math_random(4,8)}, weight = 3, evolution_min = 0.2, evolution_max = 0.7},
		--{{name = "fast-splitter", count = math_random(2,4)}, weight = 3, evolution_min = 0.2, evolution_max = 0.3},
		--{{name = "transport-belt", count = math_random(25,75)}, weight = 3, evolution_min = 0, evolution_max = 0.3},
		--{{name = "underground-belt", count = math_random(4,8)}, weight = 3, evolution_min = 0, evolution_max = 0.3},
		--{{name = "splitter", count = math_random(2,4)}, weight = 3, evolution_min = 0, evolution_max = 0.3},		
		--{{name = "oil-refinery", count = math_random(2,4)}, weight = 2, evolution_min = 0.3, evolution_max = 1},
		--{{name = "pipe", count = math_random(30,50)}, weight = 3, evolution_min = 0.0, evolution_max = 0.3},
		--{{name = "pipe-to-ground", count = math_random(4,8)}, weight = 1, evolution_min = 0.2, evolution_max = 0.5},
		--{{name = "pumpjack", count = math_random(1,3)}, weight = 1, evolution_min = 0.3, evolution_max = 0.8},
		--{{name = "pump", count = math_random(1,2)}, weight = 1, evolution_min = 0.3, evolution_max = 0.8},
		--{{name = "solar-panel", count = math_random(8,16)}, weight = 3, evolution_min = 0.4, evolution_max = 0.9},
		{{name = "electric-furnace", count = math_random(2,4)}, weight = 3, evolution_min = 0.5, evolution_max = 1},
		--{{name = "steel-furnace", count = math_random(4,8)}, weight = 3, evolution_min = 0.2, evolution_max = 0.7},
		--{{name = "stone-furnace", count = math_random(8,16)}, weight = 3, evolution_min = 0.0, evolution_max = 0.1},		
		--{{name = "radar", count = math_random(1,2)}, weight = 1, evolution_min = 0.1, evolution_max = 0.3},
		--{{name = "rail-signal", count = math_random(8,16)}, weight = 2, evolution_min = 0.2, evolution_max = 0.8},
		--{{name = "rail-chain-signal", count = math_random(8,16)}, weight = 2, evolution_min = 0.2, evolution_max = 0.8},		
		--{{name = "stone-wall", count = math_random(25,75)}, weight = 1, evolution_min = 0.1, evolution_max = 0.5},
		--{{name = "gate", count = math_random(4,8)}, weight = 1, evolution_min = 0.1, evolution_max = 0.5},
		--{{name = "storage-tank", count = math_random(1,4)}, weight = 3, evolution_min = 0.3, evolution_max = 0.6},
		--{{name = "train-stop", count = math_random(1,2)}, weight = 1, evolution_min = 0.2, evolution_max = 0.7},
		{{name = "express-loader", count = math_random(1,3)}, weight = 1, evolution_min = 0.5, evolution_max = 1},
		{{name = "fast-loader", count = math_random(1,3)}, weight = 1, evolution_min = 0.2, evolution_max = 0.7},
		--{{name = "loader", count = math_random(1,3)}, weight = 1, evolution_min = 0.0, evolution_max = 0.5},
		--{{name = "lab", count = math_random(1,2)}, weight = 2, evolution_min = 0.0, evolution_max = 0.1},	
		{{name = "roboport", count = math_random(2,4)}, weight = 2, evolution_min = 0.6, evolution_max = 1},
		--{{name = "flamethrower-turret", count = math_random(1,3)}, weight = 3, evolution_min = 0.5, evolution_max = 1},		
		--{{name = "laser-turret", count = math_random(4,8)}, weight = 3, evolution_min = 0.5, evolution_max = 1},	
		--{{name = "gun-turret", count = math_random(2,6)}, weight = 3, evolution_min = 0.2, evolution_max = 0.9},

		{{name = "coin", count = math_random(1000,2500)}, weight = 1, evolution_min = 0.1, evolution_max = 1}	
	}

	for _, t in pairs (chest_loot_BLACKMARKET3) do
		for x = 1, t.weight, 1 do
			table.insert(chest_raffle_BLACKMARKET3, t[1])
		end			
	end
	
	local shipBM3 = surface.create_entity{name = "big-ship-wreck-1", position = {BlackMarket3Pos.x-18, BlackMarket3Pos.y+6}, force = force}
	shipBM3.minable = false
	local i = shipBM3.get_inventory(defines.inventory.chest)
	for x = 1, math_random(3,5), 1 do
		local loot = chest_raffle_BLACKMARKET3[math_random(1,#chest_raffle_BLACKMARKET3)]
		i.insert(loot)
	end
end


function market3wall(surface, BlackMarket3Pos, force)
    
	local wall1 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-11, BlackMarket3Pos.y-11}, force = force}
    wall1.minable = false
	local wall2 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-12, BlackMarket3Pos.y-11}, force = force}
    wall2.minable = false
	local wall3 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-13, BlackMarket3Pos.y-11}, force = force}
    wall3.minable = false
	local wall4 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-14, BlackMarket3Pos.y-11}, force = force}
    wall4.minable = false
	local wall5 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-15, BlackMarket3Pos.y-11}, force = force}
    wall5.minable = false
	local wall6 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-16, BlackMarket3Pos.y-11}, force = force}
    wall6.minable = false
	local wall7 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y-11}, force = force}
    wall7.minable = false
	local wall8 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y-10}, force = force}
    wall8.minable = false
	local wall8 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y-9}, force = force}
    wall8.minable = false
	local wall9 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y-8}, force = force}
    wall9.minable = false
	local wall10 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y-7}, force = force}
    wall10.minable = false
	local wall11 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y-6}, force = force}
    wall11.minable = false
	local wall12 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y-5}, force = force}
    wall12.minable = false
	local wall13 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y-4}, force = force}
    wall13.minable = false
	local wall14 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y-3}, force = force}
    wall14.minable = false
	local wall15 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y-2}, force = force}
    wall15.minable = false
	local wall16 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y-1}, force = force}
    wall16.minable = false
	local wall17 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y-0}, force = force}
    wall17.minable = false
	local wall18 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y+1}, force = force}
    wall18.minable = false
	local wall19 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y+2}, force = force}
    wall19.minable = false
	local wall20 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y+3}, force = force}
    wall20.minable = false
	local wall21 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y+4}, force = force}
    wall21.minable = false
	local wall22 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y+5}, force = force}
    wall22.minable = false
	local wall23 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y+6}, force = force}
    wall23.minable = false
	local wall24 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y+7}, force = force}
    wall24.minable = false
	local wall25 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-17, BlackMarket3Pos.y+8}, force = force}
    wall25.minable = false
	local wall26 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-16, BlackMarket3Pos.y+8}, force = force}
    wall26.minable = false
	local wall27 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-15, BlackMarket3Pos.y+8}, force = force}
    wall27.minable = false
	local wall28 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-14, BlackMarket3Pos.y+8}, force = force}
    wall28.minable = false
	local wall29 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-13, BlackMarket3Pos.y+8}, force = force}
    wall29.minable = false
	local wall30 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-12, BlackMarket3Pos.y+8}, force = force}
    wall30.minable = false
	local wall31 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-11, BlackMarket3Pos.y+8}, force = force}
    wall31.minable = false
	local wall32 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-10, BlackMarket3Pos.y+8}, force = force}
    wall32.minable = false
	local wall33 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-9, BlackMarket3Pos.y+8}, force = force}
    wall33.minable = false
	local wall34 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-8, BlackMarket3Pos.y+8}, force = force}
    wall34.minable = false
	local wall35 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-7, BlackMarket3Pos.y+8}, force = force}
    wall35.minable = false
	local wall36 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-6, BlackMarket3Pos.y+8}, force = force}
    wall36.minable = false
	local wall37 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-5, BlackMarket3Pos.y+8}, force = force}
    wall37.minable = false
	local wall38 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-4, BlackMarket3Pos.y+8}, force = force}
    wall38.minable = false
	local wall39 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-3, BlackMarket3Pos.y+8}, force = force}
    wall39.minable = false
	local wall40 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-2, BlackMarket3Pos.y+8}, force = force}
    wall40.minable = false
	local wall41 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+2, BlackMarket3Pos.y+8}, force = force}
    wall41.minable = false
	local wall42 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+3, BlackMarket3Pos.y+8}, force = force}
    wall42.minable = false
	local wall43 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+4, BlackMarket3Pos.y+8}, force = force}
    wall43.minable = false
	local wall44 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+5, BlackMarket3Pos.y+8}, force = force}
    wall44.minable = false
	local wall45 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+6, BlackMarket3Pos.y+8}, force = force}
    wall45.minable = false
	local wall46 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+7, BlackMarket3Pos.y+8}, force = force}
    wall46.minable = false
	local wall48 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+8, BlackMarket3Pos.y+8}, force = force}
    wall48.minable = false
	local wall49 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+9, BlackMarket3Pos.y+8}, force = force}
    wall49.minable = false
	local wall50 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+10, BlackMarket3Pos.y+8}, force = force}
    wall50.minable = false
	local wall51 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+11, BlackMarket3Pos.y+8}, force = force}
    wall51.minable = false
	local wall52 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y+8}, force = force}
    wall52.minable = false
	local wall53 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y+7}, force = force}
    wall53.minable = false
	local wall54 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y+6}, force = force}
    wall54.minable = false
	local wall55 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y+5}, force = force}
    wall55.minable = false
	local wall56 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y+4}, force = force}
    wall56.minable = false
	local wall57 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y+3}, force = force}
    wall57.minable = false
	local wall58 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y+2}, force = force}
    wall58.minable = false
	local wall59 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y+1}, force = force}
    wall59.minable = false
	local wall60 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y-0}, force = force}
    wall60.minable = false
	local wall61 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y-1}, force = force}
    wall61.minable = false
	local wall62 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y-2}, force = force}
    wall62.minable = false
	local wall63 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y-3}, force = force}
    wall63.minable = false
	local wall64 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y-4}, force = force}
    wall64.minable = false
	local wall65 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y-5}, force = force}
    wall65.minable = false
	local wall66 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y-6}, force = force}
    wall66.minable = false
	local wall67 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y-7}, force = force}
    wall67.minable = false
	local wall68 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y-8}, force = force}
    wall68.minable = false
	local wall69 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y-9}, force = force}
    wall69.minable = false
	local wall70 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y-10}, force = force}
    wall70.minable = false
	local wall71 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y-11}, force = force}
    wall71.minable = false
	local wall72 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+11, BlackMarket3Pos.y-11}, force = force}
    wall72.minable = false
	local wall73 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+10, BlackMarket3Pos.y-11}, force = force}
    wall73.minable = false
	local wall74 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+9, BlackMarket3Pos.y-11}, force = force}
    wall74.minable = false
	local wall75 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+8, BlackMarket3Pos.y-11}, force = force}
    wall75.minable = false
	local wall76 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+7, BlackMarket3Pos.y-11}, force = force}
    wall76.minable = false
	local wall77 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+6, BlackMarket3Pos.y-11}, force = force}
    wall77.minable = false
	local wall78 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+5, BlackMarket3Pos.y-11}, force = force}
    wall78.minable = false
	local wall79 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+4, BlackMarket3Pos.y-11}, force = force}
    wall79.minable = false
	local wall80 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+3, BlackMarket3Pos.y-11}, force = force}
    wall80.minable = false
	local wall81 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+2, BlackMarket3Pos.y-11}, force = force}
    wall81.minable = false
	local wall82 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x+1, BlackMarket3Pos.y-11}, force = force}
    wall82.minable = false
	local wall83 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-0, BlackMarket3Pos.y-11}, force = force}
    wall83.minable = false
	local wall84 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-1, BlackMarket3Pos.y-11}, force = force}
    wall84.minable = false
	local wall85 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-2, BlackMarket3Pos.y-11}, force = force}
    wall85.minable = false
	local wall86 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-3, BlackMarket3Pos.y-11}, force = force}
    wall86.minable = false
	local wall87 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-4, BlackMarket3Pos.y-11}, force = force}
    wall87.minable = false
	local wall88 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-5, BlackMarket3Pos.y-11}, force = force}
    wall88.minable = false
	local wall89 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-6, BlackMarket3Pos.y-11}, force = force}
    wall89.minable = false
	local wall90 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-7, BlackMarket3Pos.y-11}, force = force}
    wall90.minable = false
	local wall91 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-8, BlackMarket3Pos.y-11}, force = force}
    wall91.minable = false
	local wall92 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-9, BlackMarket3Pos.y-11}, force = force}
    wall92.minable = false
	local wall93 = surface.create_entity{name = "stone-wall", position = {BlackMarket3Pos.x-10, BlackMarket3Pos.y-11}, force = force}
    wall93.minable = false
end

function market3light(surface, BlackMarket3Pos, force)

    local solar1 = surface.create_entity{name = "solar-panel", position = {BlackMarket3Pos.x-10, BlackMarket3Pos.y-4}, force = force}
    solar1.minable = false
    local solar2 = surface.create_entity{name = "solar-panel", position = {BlackMarket3Pos.x-10, BlackMarket3Pos.y+1}, force = force}
    solar2.minable = false
    local pole1 = surface.create_entity{name = "medium-electric-pole", position = {BlackMarket3Pos.x-9, BlackMarket3Pos.y-1}}
    pole1.minable = false
	local pole2 = surface.create_entity{name = "medium-electric-pole", position = {BlackMarket3Pos.x-2, BlackMarket3Pos.y-1}}
    pole2.minable = false
	local pole3 = surface.create_entity{name = "medium-electric-pole", position = {BlackMarket3Pos.x+2, BlackMarket3Pos.y-1}}
    pole3.minable = false
    local akku = surface.create_entity{name = "accumulator", position = {BlackMarket3Pos.x-10, BlackMarket3Pos.y-1}, force = force}
    akku.minable = false
	local lamp = surface.create_entity{name = "small-lamp", position = {BlackMarket3Pos.x-3, BlackMarket3Pos.y-2}, force = force}
    lamp.minable = false
	local lamp2 = surface.create_entity{name = "small-lamp", position = {BlackMarket3Pos.x+3, BlackMarket3Pos.y+2}, force = force}
    lamp2.minable = false
	local lamp3 = surface.create_entity{name = "small-lamp", position = {BlackMarket3Pos.x-3, BlackMarket3Pos.y+2}, force = force}
    lamp3.minable = false
	local lamp4 = surface.create_entity{name = "small-lamp", position = {BlackMarket3Pos.x+3, BlackMarket3Pos.y-2}, force = force}
    lamp4.minable = false
end

function market3biter(surface, BlackMarket3Pos)
 
	local spawner1 = surface.create_entity{name = "biter-spawner", position = {BlackMarket3Pos.x+5, BlackMarket3Pos.y-2}}
	local spawner2 = surface.create_entity{name = "biter-spawner", position = {BlackMarket3Pos.x+9, BlackMarket3Pos.y+2}}
	local spawner3 = surface.create_entity{name = "biter-spawner", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y-4}}
	local spawner4 = surface.create_entity{name = "biter-spawner", position = {BlackMarket3Pos.x+15, BlackMarket3Pos.y-9}}
	local spawner5 = surface.create_entity{name = "biter-spawner", position = {BlackMarket3Pos.x-18, BlackMarket3Pos.y-15}}
	local spawner6 = surface.create_entity{name = "biter-spawner", position = {BlackMarket3Pos.x-11, BlackMarket3Pos.y-18}}
	local spawner7 = surface.create_entity{name = "biter-spawner", position = {BlackMarket3Pos.x-14, BlackMarket3Pos.y-21}}
	local spawner8 = surface.create_entity{name = "biter-spawner", position = {BlackMarket3Pos.x-10, BlackMarket3Pos.y-2}}
	local spawner9 = surface.create_entity{name = "biter-spawner", position = {BlackMarket3Pos.x-6, BlackMarket3Pos.y+10}}
	local spawner10 = surface.create_entity{name = "biter-spawner", position = {BlackMarket3Pos.x-1, BlackMarket3Pos.y+11}}
	local spawner11 = surface.create_entity{name = "biter-spawner", position = {BlackMarket3Pos.x+4, BlackMarket3Pos.y+14}}
	local spawner12 = surface.create_entity{name = "spitter-spawner", position = {BlackMarket3Pos.x-9, BlackMarket3Pos.y-8}}
	local spawner13 = surface.create_entity{name = "spitter-spawner", position = {BlackMarket3Pos.x-5, BlackMarket3Pos.y-6}}
	local spawner14 = surface.create_entity{name = "spitter-spawner", position = {BlackMarket3Pos.x+9, BlackMarket3Pos.y+15}}
	local spawner15 = surface.create_entity{name = "spitter-spawner", position = {BlackMarket3Pos.x-0, BlackMarket3Pos.y-14}}
	local spawner16 = surface.create_entity{name = "spitter-spawner", position = {BlackMarket3Pos.x-6, BlackMarket3Pos.y-15}}
	local worm1 = surface.create_entity{name = "medium-worm-turret", position = {BlackMarket3Pos.x-6, BlackMarket3Pos.y+2}}
	local worm2 = surface.create_entity{name = "medium-worm-turret", position = {BlackMarket3Pos.x-5, BlackMarket3Pos.y-1}}
	local worm3 = surface.create_entity{name = "medium-worm-turret", position = {BlackMarket3Pos.x+1, BlackMarket3Pos.y-6}}
	local worm4 = surface.create_entity{name = "medium-worm-turret", position = {BlackMarket3Pos.x-12, BlackMarket3Pos.y-10}}
	local worm5 = surface.create_entity{name = "medium-worm-turret", position = {BlackMarket3Pos.x-10, BlackMarket3Pos.y-12}}
	local worm6 = surface.create_entity{name = "medium-worm-turret", position = {BlackMarket3Pos.x+9, BlackMarket3Pos.y+12}}
	local worm7 = surface.create_entity{name = "medium-worm-turret", position = {BlackMarket3Pos.x+4, BlackMarket3Pos.y+11}}
end

function market3wreckage(surface, BlackMarket3Pos, force)
    local wreckage1 = surface.create_entity{name = "mineable-wreckage", position = {BlackMarket3Pos.x-11, BlackMarket3Pos.y+12}, force = force}
    local wreckage2 = surface.create_entity{name = "mineable-wreckage", position = {BlackMarket3Pos.x-10, BlackMarket3Pos.y+15}, force = force}
    local wreckage3 = surface.create_entity{name = "mineable-wreckage", position = {BlackMarket3Pos.x-9, BlackMarket3Pos.y-13}, force = force}
	local wreckage4 = surface.create_entity{name = "mineable-wreckage", position = {BlackMarket3Pos.x-12, BlackMarket3Pos.y-14}, force = force}
	local wreckage5 = surface.create_entity{name = "mineable-wreckage", position = {BlackMarket3Pos.x+12, BlackMarket3Pos.y-10}, force = force}
    local wreckage6 = surface.create_entity{name = "mineable-wreckage", position = {BlackMarket3Pos.x-14, BlackMarket3Pos.y-1}, force = force}
	local wreckage7 = surface.create_entity{name = "mineable-wreckage", position = {BlackMarket3Pos.x-13, BlackMarket3Pos.y-12}, force = force}
	local wreckage8 = surface.create_entity{name = "mineable-wreckage", position = {BlackMarket3Pos.x+13, BlackMarket3Pos.y+12}, force = force}
	local wreckage9 = surface.create_entity{name = "mineable-wreckage", position = {BlackMarket3Pos.x-13, BlackMarket3Pos.y+12}, force = force}
	local wreckage10 = surface.create_entity{name = "mineable-wreckage", position = {BlackMarket3Pos.x-18, BlackMarket3Pos.y+2}, force = force}
	local wreckage11 = surface.create_entity{name = "mineable-wreckage", position = {BlackMarket3Pos.x-16, BlackMarket3Pos.y+4}, force = force}
end