-- control.lua
-- Mar 2019

-- Oarc's Separated Spawn Scenario
-- 
-- I wanted to create a scenario that allows you to spawn in separate locations
-- From there, I ended up adding a bunch of other minor/major features
-- 
-- Credit:
--  Tags - Taken from WOGs scenario 
--  Rocket Silo - Taken from Frontier as an idea
--
-- Feel free to re-use anything you want. It would be nice to give me credit
-- if you can.

-- Generic Utility Includes
require("lib/oarc_utils")

-- Other soft-mod type features.
require("lib/frontier_silo")
require("lib/tag")
require("lib/game_opts")
require("lib/regrowth_map")
require("lib/player_list")
require("lib/rocket_launch")

-- MODULES by MewMew (Comfy)
require("modules/railgun_enhancer")
require("modules/rocks_yield_ore")
require("modules/junk_yield_ore_veins")
require("modules/mineable_wreckage_yields_scrap")
require("modules/dynamic_landfill")
require("modules/biters_yield_coins")
require("modules/biters_double_damage")
require("modules/biters_double_hp")
require("modules/spawners_double_hp")
require("modules/spawners_contain_biters")
require("modules/splice")


-- Black market by Hansi-TWZ - DE333-SERVER.EU
require("DE333/market/black_market")
require("DE333/market/black_market2")
require("DE333/market/black_market3")

-- Crashsites & wreckage by Hansi-TWZ - DE333-SERVER.EU
require("DE333/crashsite")
require("DE333/wreckage_spawner")

-- Main Configuration File
require("config")
require("DE333/DE333_config")

-- Save all config settings to global table.
require("lib/oarc_global_cfg.lua")

-- Scenario Specific Includes
require("lib/separate_spawns")
require("lib/separate_spawns_guis")

require("DE333/maps/ToxiPlanet")
require("DE333/maps/Junkyard")
require("DE333/maps/RedDesert")
require("DE333/maps/Woody")

-- Create a new surface so we can modify map settings at the start.
GAME_SURFACE_NAME="DE333"

--------------------------------------------------------------------------------
-- ALL EVENT HANLDERS ARE HERE IN ONE PLACE!
--------------------------------------------------------------------------------

----------------------------------------
-- On Init - only runs once the first 
--   time the game starts
----------------------------------------
script.on_init(function(event)

    -- FIRST
    InitOarcConfig()

    -- Create new game surface
    CreateGameSurface()

    -- MUST be before other stuff, but after surface creation.
    InitSpawnGlobalsAndForces()

    if SILO_FIXED_POSITION then
        SetFixedSiloPosition(SILO_POSITION)
    else
        SetRandomSiloPosition(SILO_NUM_SPAWNS)
    end

    if FRONTIER_ROCKET_SILO_MODE then
        GenerateRocketSiloAreas(game.surfaces[GAME_SURFACE_NAME])
    end
	
	if ENABLE_BLACKMARKET1 then
		SetRandomBlackMarket1Position(BLACKMARKET1_NUM_SPAWNS)
		GenerateBlackMarket1Areas(game.surfaces[GAME_SURFACE_NAME])
	end
	
	if ENABLE_BLACKMARKET2 then
		SetRandomBlackMarket2Position(BLACKMARKET2_NUM_SPAWNS)
		GenerateBlackMarket2Areas(game.surfaces[GAME_SURFACE_NAME])
	end
	
	if ENABLE_BLACKMARKET3 then
		SetRandomBlackMarket3Position(BLACKMARKET3_NUM_SPAWNS)
		GenerateBlackMarket3Areas(game.surfaces[GAME_SURFACE_NAME])
	end
	
    -- Regardless of whether it's enabled or not, it's good to have this init.
    OarcRegrowthInit()
	on_init_on_tick_shedule(event)
end)


----------------------------------------
-- Freeplay rocket launch info
-- Slightly modified for my purposes
----------------------------------------
script.on_event(defines.events.on_rocket_launched, function(event)
    if global.ocfg.frontier_rocket_silo then
        RocketLaunchEvent(event)
    end
end)


local first_chunk_generated_flag = false
----------------------------------------
-- Chunk Generation
----------------------------------------
script.on_event(defines.events.on_chunk_generated, function(event)

    if global.ocfg.enable_regrowth then
        OarcRegrowthChunkGenerate(event.area.left_top)
    end

    if global.ocfg.enable_undecorator then
        UndecorateOnChunkGenerate(event)
    end

    if global.ocfg.frontier_rocket_silo then
        GenerateRocketSiloChunk(event)
    end
	
	if ENABLE_BLACKMARKET1 then
		GenerateBlackMarket1Chunk(event)
	end
	
	if ENABLE_BLACKMARKET2 then
		GenerateBlackMarket2Chunk(event)
	end
	
	if ENABLE_BLACKMARKET3 then
		GenerateBlackMarket3Chunk(event)
	end

    SeparateSpawnsGenerateChunk(event)

    CreateHoldingPen(event.surface, event.area, 16, 32)
	
	if ENABLE_WRECKAGE_SPAWNER then
		wreckage_spawner(event)
	end
	
	if ENABLE_CRASHSITES then
		crashsite(event)
	end
	
	if MAP_TOXIPLANET then
		ToxiPlanet(event)
	end
	
	if MAP_JUNKYARD then
		Junkyard(event)
	end
	
	if MAP_REDDESERT then
		RedDesert(event)
	end
	
	if MAP_WOODY then
		Woody(event)
	end
end)


----------------------------------------
-- Gui Click
----------------------------------------
script.on_event(defines.events.on_gui_click, function(event)
    if global.ocfg.enable_tags then
        TagGuiClick(event)
    end

    if global.ocfg.enable_player_list then
        PlayerListGuiClick(event)
    end

    WelcomeTextGuiClick(event)
    SpawnOptsGuiClick(event)
    SpawnCtrlGuiClick(event)
    SharedSpwnOptsGuiClick(event)
    BuddySpawnOptsGuiClick(event)
    BuddySpawnWaitMenuClick(event)
    BuddySpawnRequestMenuClick(event)
    SharedSpawnJoinWaitMenuClick(event)

    GameOptionsGuiClick(event)
	RocketGuiClick(event)

end)

script.on_event(defines.events.on_gui_checked_state_changed, function (event)
    SpawnOptsRadioSelect(event)
    SpawnCtrlGuiOptionsSelect(event)
end)


----------------------------------------
-- Player Events
----------------------------------------
script.on_event(defines.events.on_player_joined_game, function(event)

    CreateGameOptionsGui(event)

    PlayerJoinedMessages(event)

    if global.ocfg.enable_player_list then
        CreatePlayerListGui(event)
    end

    if global.ocfg.enable_tags then
        CreateTagGui(event)
    end
	
	if global.satellite_sent then
        CreateRocketGui(game.players[event.player_index])
    end
	
	if BITERS_DOUBLE_DAMAGE then
        biters_double_damage(event)
    end

end)

script.on_event(defines.events.on_player_created, function(event)
    
    -- Move the player to the game surface immediately.
    -- May change this to Lobby in the future.
    game.players[event.player_index].teleport({x=0,y=0}, GAME_SURFACE_NAME)

    if global.ocfg.enable_long_reach then
        GivePlayerLongReach(game.players[event.player_index])
    end

    SeparateSpawnsPlayerCreated(event.player_index)
end)

script.on_event(defines.events.on_player_respawned, function(event)
    SeparateSpawnsPlayerRespawned(event)
   
    PlayerRespawnItems(event)

    if global.ocfg.enable_long_reach then
        GivePlayerLongReach(game.players[event.player_index])
    end
end)

script.on_event(defines.events.on_player_left_game, function(event)
    FindUnusedSpawns(event)
end)

script.on_event(defines.events.on_built_entity, function(event)
    if global.ocfg.enable_autofill then
        Autofill(event)
    end

    if global.ocfg.enable_regrowth then
        OarcRegrowthOffLimits(event.created_entity.position, 2)
    end

    if ENABLE_ANTI_GRIEFING then
        SetItemBlueprintTimeToLive(event)
    end
end)


----------------------------------------
-- Shared vision, charts a small area around other players
----------------------------------------
script.on_event(defines.events.on_tick, function(event)
    if global.ocfg.enable_regrowth then
        OarcRegrowthOnTick()
    end

    if global.ocfg.enable_abandoned_base_removal then
        OarcRegrowthForceRemovalOnTick()
    end

    DelayedSpawnOnTick()

    if global.ocfg.frontier_rocket_silo then
        DelayedSiloCreationOnTick(game.surfaces[GAME_SURFACE_NAME])
    end
	
	if ENABLE_BLACKMARKET1 then
		DelayedBlackMarket1CreationOnTick(game.surfaces[GAME_SURFACE_NAME])
	end
	
	if ENABLE_BLACKMARKET2 then
		DelayedBlackMarket2CreationOnTick(game.surfaces[GAME_SURFACE_NAME])
	end
	
	if ENABLE_BLACKMARKET3 then
		DelayedBlackMarket3CreationOnTick(game.surfaces[GAME_SURFACE_NAME])
	end
	
	on_tick_shedule(event)
end)



script.on_event(defines.events.on_sector_scanned, function (event)
    if global.ocfg.enable_regrowth then
        OarcRegrowthSectorScan(event)
    end
end)

----------------------------------------
-- Refreshes regrowth timers around an active timer
-- Refresh areas where stuff is built, and mark any chunks with player
-- built stuff as permanent.
----------------------------------------
script.on_event(defines.events.on_robot_built_entity, function (event)
    if global.ocfg.enable_regrowth then
        OarcRegrowthOffLimits(event.created_entity.position, 2)
    end
end)
-- I disabled this because it's too much overhead for too little gain!
-- script.on_event(defines.events.on_player_mined_entity, function(event)
--     if global.ocfg.enable_regrowth then
--         OarcRegrowthCheckChunkEmpty(event)
--     end
-- end)
-- script.on_event(defines.events.on_robot_mined_entity, function(event)
--     if global.ocfg.enable_regrowth then
--         OarcRegrowthCheckChunkEmpty(event)
--     end
-- end)

----------------------------------------
-- Shared chat, so you don't have to type /s
----------------------------------------
script.on_event(defines.events.on_console_chat, function(event)
    if (global.ocfg.enable_shared_chat) then
        if (event.player_index ~= nil) then
            ShareChatBetweenForces(game.players[event.player_index], event.message)
        end
    end
end)

local loaders_technology_map = {
    ['logistics'] = 'loader',
    ['logistics-2'] = 'fast-loader',
    ['logistics-3'] = 'express-loader'
}

local railgun_technology_map = {
    ['laser-turrets'] = 'railgun'
}

local railgun_dart_technology_map = {
    ['laser-turrets'] = 'railgun-dart'
}

----------------------------------------
-- On Research Finished
-- This is where you can permanently remove researched techs
----------------------------------------
script.on_event(defines.events.on_research_finished, function(event)
    if global.ocfg.frontier_rocket_silo then
        RemoveRecipe(event.research.force, "rocket-silo")
    end
	if LOADERS_ENABLED then
        local research = event.research
        local recipe = loaders_technology_map[research.name]
        if recipe then
            research.force.recipes[recipe].enabled = true
        end
    end
	if RAILGUN_ENABLED then
        local research = event.research
        local recipe = railgun_technology_map[research.name]
        if recipe then
            research.force.recipes[recipe].enabled = true
        end
    end
	if RAILGUN_ENABLED then
        local research = event.research
        local recipe = railgun_dart_technology_map[research.name]
        if recipe then
            research.force.recipes[recipe].enabled = true
        end
    end
	if LOCK_GOODIES_UNTIL_ROCKET_LAUNCH and 
        (not global.satellite_sent or not global.satellite_sent[event.research.force.name]) then
        RemoveRecipe(event.research.force, "productivity-module-3")
        RemoveRecipe(event.research.force, "speed-module-3")
    end
end)

----------------------------------------
-- On Entity Spawned
-- This is where I modify biter spawning based on location and other factors.
----------------------------------------
script.on_event(defines.events.on_entity_spawned, function(event)
    if (global.ocfg.modified_enemy_spawning) then
        ModifyEnemySpawnsNearPlayerStartingAreas(event)
    end
end)
-- on_biter_base_built -- Worth considering for later.

----------------------------------------
-- On Corpse Timed Out
-- Save player's stuff so they don't lose it if they can't get to the corpse fast enough.
----------------------------------------
script.on_event(defines.events.on_character_corpse_expired, function(event)
    DropGravestoneChestFromCorpse(event.corpse)
end)


script.on_event(defines.events.on_entity_damaged, function(event)
	if BITERS_DOUBLE_HP then
		biters_double_hp(event)
	end
	if SPAWNERS_DOUBLE_HP then
		spawners_double_hp(event)
	end
	if RAILGUN_ENABLED then
		railgun_enhancer(event)
	end
end)

script.on_event(defines.events.on_player_mined_entity, function(event)
	if MINEABLE_WRECKAGE_YIELDS_SCRAP then
		mineable_wreckage_yields_scrap(event)
	end
	
	if ROCKS_YIELD_ORE then
		rocks_yield_ore_on_player(event)
	end
	
	if UNEARTHING_BITERS then
		UnearthingBiters(event)
	end
	
	if JUNK_YIELD_ORE_VEINS_ENABLED then
		JunkYieldOreVeins(event)
	end
end)

script.on_event(defines.events.on_entity_died, function(event)
	if BITERS_YIELD_COINS then
		biters_yield_coins(event)
	end
	if SPAWNERS_CONTAIN_BITERS then
		spawners_contain_biters(event)
	end
	if SPLICE then
		splice(event)
	end
	if ROCKS_YIELD_ORE then
		rocks_yield_ore_on_entity_died(event)
	end
end)

script.on_event(defines.events.on_player_built_tile, function(event)
	if DYNAMIC_LANDFILL then
		DynamicLandfillPlayer(event)
	end
end)

script.on_event(defines.events.on_robot_built_tile, function(event)
	if DYNAMIC_LANDFILL then
		DynamicLandfillRobot(event)
	end
end)

script.on_event(defines.events.on_marked_for_deconstruction, function(event)
	if DISABLED_DECONSTRUCTION_ENABLE then
		DisabledDeconstruction(event)
	end

end)