-- game_opts.lua
-- Jan 2018
-- Display current game options, maybe have some admin controls here

-- Main Configuration File
require("config")
require("lib/oarc_utils")
require("lib/separate_spawns")

function CreateGameOptionsGui(event)
    local player = game.players[event.player_index]
    if player.gui.top.game_options == nil then
        player.gui.top.add{name="game_options", type="button", caption="Info"}
    end   
end

local function ExpandGameOptionsGui(player)
    local frame = player.gui.left["game_options_panel"]
    if (frame) then
        frame.destroy()
    else
        local frame = player.gui.left.add{type="frame",
                                            name="game_options_panel", 
                                            caption="Server Info:",
                                            direction="vertical"}

        -- General Server Info:
        AddLabel(frame, "info_1", global.ocfg.welcome_msg, my_longer_label_style)
        AddLabel(frame, "info_2", global.ocfg.server_rules, my_longer_label_style)
        AddLabel(frame, "info_3", global.ocfg.server_contact, my_longer_label_style)
        AddSpacer(frame, "info_spacer1")

        -- Enemy Settings:
        local enemy_expansion_txt = "disabled"
        if game.map_settings.enemy_expansion.enabled then enemy_expansion_txt = "enabled" end

        local enemy_text="Server Run Time: " .. formattime_hours_mins(game.tick) .. "\n" ..
        "Current Evolution: " .. string.format("%.4f", game.forces["enemy"].evolution_factor) .. "\n" ..
        "Enemy expansion is " .. enemy_expansion_txt

        AddLabel(frame, "enemy_info", enemy_text, my_longer_label_style)
        AddSpacer(frame, "enemy_info_spacer1")

        -- Soft Mods:
        local soft_mods_string = "[font=default-dropdown]Oarc Core[/font]"
        if (global.ocfg.enable_undecorator) then
            soft_mods_string = soft_mods_string .. "[font=default-dropdown], Undecorator[/font]"
        end
        if (global.ocfg.enable_tags) then
            soft_mods_string = soft_mods_string .. "[font=default-dropdown], Tags[/font]"
        end
        if (global.ocfg.enable_long_reach) then
            soft_mods_string = soft_mods_string .. "[font=default-dropdown], Long Reach[/font]"
        end
        if (global.ocfg.enable_autofill) then
            soft_mods_string = soft_mods_string .. "[font=default-dropdown], Auto Fill[/font]"
        end
        if (global.ocfg.enable_player_list) then
            soft_mods_string = soft_mods_string .. "[font=default-dropdown], Player List[/font]"
        end		
		if (LOADERS_ENABLED) then
            soft_mods_string = soft_mods_string .. "[font=default-dropdown], Loaders[/font]"
        end
		if (RAILGUN_ENABLED) then
            soft_mods_string = soft_mods_string .. "[font=default-dropdown], Railgun[/font]"
        end
		if (ENABLE_MARKET) then
            soft_mods_string = soft_mods_string .. "[font=default-dropdown], Market[/font]"
        end
		if (ENABLE_BLACKMARKET1) or (ENABLE_BLACKMARKET2) or (ENABLE_BLACKMARKET3)then
            soft_mods_string = soft_mods_string .. "[font=default-dropdown], Black Market[/font]"
        end
		if (ENABLE_CRASHSITES) then
            soft_mods_string = soft_mods_string .. "[font=default-dropdown], Crashsites[/font]"
        end

        local game_info_str = "[font=default-dropdown]Soft Mods Enabled: [/font]" .. soft_mods_string

        -- Spawn options:
        if (global.ocfg.enable_separate_teams) then
			game_info_str = game_info_str.."\n"..""
            game_info_str = game_info_str.."\n".."[font=default-dropdown]You are allowed to spawn on your own team (have your own research tree). All teams are friendly![/font]"
        end
        if (global.ocfg.enable_vanilla_spawns) then
            game_info_str = game_info_str.."\n".."[font=default-dropdown]You are spawned in a default style starting area.[/font]"
        else
            game_info_str = game_info_str.."\n".."[font=default-dropdown]You are spawned with a fix set of starting resources.[/font]"
            if (global.ocfg.enable_buddy_spawn) then
                game_info_str = game_info_str.."\n".."[font=default-dropdown]You can chose to spawn alongside a buddy if you spawn together at the same time.[/font]"
            end
        end
        if (global.ocfg.enable_shared_spawns) then
            game_info_str = game_info_str.."\n".."[font=default-dropdown]Spawn hosts may choose to share their spawn and allow other players to join them.[/font]"
        end
        if (global.ocfg.enable_separate_teams and global.ocfg.enable_shared_team_vision) then
            game_info_str = game_info_str.."\n".."[font=default-dropdown]Everyone (all teams) have shared vision.[/font]"
        end
        if (global.ocfg.frontier_rocket_silo) then
            game_info_str = game_info_str.."\n".."[font=default-dropdown]Silos are NOT craftable. There are 8 already located on the map.[/font]"
        end
		if (ENABLE_BLACKMARKET1) or (ENABLE_BLACKMARKET2) or (ENABLE_BLACKMARKET3) then
            game_info_str = game_info_str.."\n".."[font=default-dropdown]There are 3 black market´s with random location on MAP.[/font]"
        end
		if (LOADERS_ENABLED) then
            game_info_str = game_info_str.."\n".."[font=default-dropdown]Loader´s available with logistic research[/font]"
        end
		if (RAILGUN_ENABLED) then
            game_info_str = game_info_str.."\n".."[font=default-dropdown]Railgun available with laser turret research[/font]"
        end
		if (ENABLE_CRASHSITES) then
            game_info_str = game_info_str.."\n".."[font=default-dropdown][color=1,0.1,0.1]!!NEW!! [/color]Crashsites with random loot spawns now on map[/font]"
        end
		if (LOCK_GOODIES_UNTIL_ROCKET_LAUNCH) then
            game_info_str = game_info_str.."\n".."[font=default-dropdown][color=1,0.1,0.1]!!NEW!! [/color]Atomic Bomb, Power Armor MkII, Artillery research available after 1st ROCKET LAUNCH.[/font]"
			game_info_str = game_info_str.."\n".."[font=default-dropdown][color=1,0.1,0.1]!!NEW!! [/color]Speed Module III, Productivity Module III recipe available after 1st ROCKET LAUNCH.[/font]"
        end
        if (global.ocfg.enable_regrowth) then
            game_info_str = game_info_str.."\n".."[font=default-dropdown]Old parts of the map will slowly be deleted over time (chunks without any player buildings).[/font]"
        end
        if (ENABLE_POWER_ARMOR_QUICK_START) then
            game_info_str = game_info_str.."\n".."[font=default-dropdown]Power armor quick start enabled.[/font]"
        end
		if (ENABLE_CREDITS) then
			game_info_str = game_info_str.."\n"..""
            game_info_str = game_info_str.."\n".."[font=default-dropdown][color=1,0.1,0.1]CREDITS:[/color][/font]"
			game_info_str = game_info_str.."\n".."[font=default-dropdown]Oarc[/font]"
			game_info_str = game_info_str.."\n".."[font=default-dropdown]MeWMeW (Comfy)[/font]"
        end

        AddLabel(frame, "game_info_label", game_info_str, my_longer_label_style)

        if (global.ocfg.enable_abandoned_base_removal) then
            AddLabel(frame, "leave_warning_msg", "If you leave within " .. global.ocfg.minimum_online_time .. " minutes of joining, your base and character will be deleted.", my_longer_label_style)
            frame.leave_warning_msg.style.font_color=my_color_red
        end

        -- Ending Spacer
        AddSpacer(frame, "end_spacer")

        -- ADMIN CONTROLS
        if (player.admin) then
            player_list = {}
            for _,player in pairs(game.connected_players) do
                table.insert(player_list, player.name)
            end
            frame.add{name = "ban_players_dropdown",
                            type = "drop-down",
                            items = player_list}
            frame.add{name="ban_player", type="button", caption="Ban Player"}
            frame.add{name="restart_player", type="button", caption="Restart Player"}
            
            -- AddLabel(frame, "restart_warning_msg", "You might CRASH the scenario by using Restart Player. Not 100% tested yet. Will not work if player has any center gui visible.", my_longer_label_style)
            -- frame.restart_warning_msg.style.font_color=my_color_red
        end
    end
end

function GameOptionsGuiClick(event) 
    if not (event and event.element and event.element.valid) then return end
    local player = game.players[event.element.player_index]
    local name = event.element.name

    if (name == "game_options") then
        ExpandGameOptionsGui(player)        
    end

    if (name == "ban_player") then
        local pIndex = event.element.parent.ban_players_dropdown.selected_index

        if (pIndex ~= 0) then
            local banPlayer = event.element.parent.ban_players_dropdown.get_item(pIndex)
            if (game.players[banPlayer]) then
                game.ban_player(banPlayer, "Banned from admin panel.")
                log("Banning " .. banPlayer)
            end
        end
    end

    if (name == "restart_player") then
        local pIndex = event.element.parent.ban_players_dropdown.selected_index

        if (pIndex ~= 0) then
            local resetPlayer = event.element.parent.ban_players_dropdown.get_item(pIndex)
            if (game.players[resetPlayer]) then
                SeparateSpawnsPlayerCreated(resetPlayer)
                log("Resetting " .. resetPlayer)
            end
        end
    end
end
